""" Zope's user folder that keeps users in MySQL tables. """

__version__ = "$Revision: 1.23 $"
# $Source: /home/vladap/src/mysqlUserFolder/RCS/mysqlUserFolder.py,v $
#
# $Author: vladap $
# $Date: 2002/03/01 14:51:31 $

import Globals, OFS, AccessControl
import time, whrandom, array, threading, base64, os, string
import cfg, util, vdb, db, log

om_class = OFS.ObjectManager.ObjectManager

# --------------------------------------------------------------------- */
# Public functions

def manage_add_mysqlUserFolder (Self, realm, use_cookies, use_sessions, 
		server, port, socket, database, username, password, 
		anon_create_ok, anon_create_role, 
		dtml_user_dir,
		cookie_path, cookie_domain,
		disable_cookies_for_ports,
		REQUEST):
	""" Add new mysqlUserFolder """
		
	if hasattr (Self.aq_base, 'acl_users'):
		return Globals.MessageDialog(
				title  ='Item Exists',
				message='This object already contains a User Folder',
				action ='%s/manage_main' % REQUEST['URL1'] )
               
	o = mysqlUserFolder (realm, server, port, socket, 
					database, username, password, 
					use_cookies, use_sessions, 
					anon_create_ok, anon_create_role, 
					cookie_path, cookie_domain,
					dtml_user_dir, disable_cookies_for_ports, REQUEST)
		
	Self._setObject ('acl_users', o)
	Self.__allow_groups__ = Self.acl_users
	
	ob = Self._getOb ('acl_users')
	ob._addRoles (ob.aq_parent)
		
	if REQUEST: return Self.manage_main (Self, REQUEST, update_menu = 1)
	
# --------------------------------------------------------------------- */
# Utility functions

def _add_dtml_method (obj, fname, id, title):
	(p, n) = os.path.split (fname)
	if string.find (n, '.') == -1:
		fname = fname + '.dtml'

	f = open (fname)
	d = f.read ()
	f.close ()
	
	obj.manage_addDTMLDocument (id, title, d)
	
def _add_role (obj, role):
	obj._addRole (role)
	
	
# --------------------------------------------------------------------- */
# Main class

class mysqlUserFolder (AccessControl.User.BasicUserFolder, 
										OFS.ObjectManager.ObjectManager):
	""" """
	
	_v_user_db = None
	_v_session_db = None
	_v_db = None
	
	meta_type = 'MySQL User Folder'
	id = 'acl_users'
	title = 'mysqlUserFolder'
	icon = 'misc_/mysqlUserFolder/folder_icon'
	
	manage_options = (
		{ 'label': 'Main',			'action': 'manage_main' },
		{ 'label': 'Contents',		'action': 'manage_contents' },
		{ 'label': 'Properties',	'action': 'manage_properties' },
		{ 'label': 'Manage Users',	'action': 'manage_users' },
		{ 'label': 'Security',		'action': 'manage_access' },
		{ 'label': 'Undo',			'action': 'manage_UndoForm' }
	)	
	
	__ac_permissions__ = (
    	('View management screens',
     		['manage','manage_menu','manage_main','manage_copyright', 
     		 'manage_tabs', 'manage_properties', 'manage_UndoForm', ]),
    	('Undo changes',			['manage_undo_transactions']),
    	('Change permissions', 		['manage_access']),
    	('Manage users', 			
    		['manage_users', 'manage_users_new', 'manage_users_edit',
    		 'manage_users_create', 'manage_users_change_other', 
    		 'manage_users_change_password', 'manage_users_delete',
			 'manage_users_change_roles',
    		 'getUserCount', 'getRoleCount', 'getUserNames', 'getRoleNames',
    		 'getUsers', 'getUser', 
    		 ]),
    	('Manage MySQL User Folders',	
    		['manage_edit', 'manage_connect',
    			'manage_addMySQLRoles', 'manage_contents']),
    )
	
	manage_properties = \
			Globals.HTMLFile ('m_Properties', globals (), mchange = 1)
	manage_main = \
			Globals.HTMLFile ('m_Contents', globals ())
	manage_users = \
			Globals.HTMLFile ('m_Users', globals ())
	manage_users_edit = \
			Globals.HTMLFile ('m_UserEdit', globals ())
	manage_users_new = \
			Globals.HTMLFile ('m_UserNew', globals ())
	manage_contents = om_class.manage_main
	
	objectIds = om_class.objectIds
	objectValues = om_class.objectValues
	objectMap = om_class.objectMap
	objectItems = om_class.objectItems
	
	def __init__ (Self, realm = '*', 
		mysql_server='localhost', mysql_port = 0,
		mysql_socket='',
		mysql_db = 'Zope', 
		mysql_user = 'Zope', mysql_password = 'Zope',
		use_cookies = 0, use_sessions = 0,
		anon_create_ok = 0, anon_create_role = '',
		cookie_path = '/', cookie_domain = '',
		dtml_user_dir = cfg.DTML_USER_DIR, 
		disable_cookies_for_ports = '',
		REQUEST = None):
		
		if not cookie_path: cookie_path = '/'
		
		Self.mysql_server = mysql_server
		Self.mysql_port = mysql_port
		Self.mysql_socket = string.strip (mysql_socket)
		Self.mysql_db = mysql_db
		Self.mysql_user = mysql_user
		Self.mysql_password = mysql_password
		
		Self.realm = realm
		Self.use_cookies = use_cookies
		Self.use_sessions = use_sessions
		
		Self.anon_create_ok = anon_create_ok
		Self.anon_create_role = anon_create_role
		
		Self.cookie_path = cookie_path
		Self.cookie_domain = cookie_domain
		
		Self.dtml_user_dir = dtml_user_dir

		Self._setupDisabledPorts (disable_cookies_for_ports)
							
		Self.__connect ()
		Self.__initUserDtml ()
		
	def __del__ (Self):
		util.log_debug ("__del__ () called ().")	
		Self.__disconnect ()
		
	def __setstate__ (Self, State):
		util.log_debug ("__setstate__ () called.")
		
		Globals.Persistent.__setstate__ (Self, State)
		
		if not hasattr (Self, 'cookie_path'):
			Self.cookie_path = '/'
			
		if not hasattr (Self, 'cookie_domain'):
			Self.cookie_domain = ''

		if not hasattr (Self, 'disable_ports'):
			Self.disable_ports = []

		if not hasattr (Self, 'mysql_socket'):
			Self.mysql_socket = ''
		
		Self.__connect ()
		
	def __initUserDtml (Self):
		pdir = util.base_prod_dir ()
		dtml_dir = pdir + '/' + Self.dtml_user_dir
		
		fname = '%s/u_Login.dtml' % dtml_dir
		_add_dtml_method (Self, fname, 'docLogin', 'User login form')
		#Self.docLogin.__roles__ = None
	
		fname = '%s/u_UserPage' % dtml_dir
		_add_dtml_method (Self, fname, 'docUserPage', "User's page")
		#Self.docUserPage.__roles__ = ["mysqlRole"]
		
		fname = '%s/u_UserNew' % dtml_dir
		_add_dtml_method (Self, fname, 'docNewUser', 'New user form')
		#Self.docNewUser.__roles__ = None
		
		fname = '%s/a_ChangeOther' % dtml_dir
		_add_dtml_method (Self, fname, 'actChangeOther', 'Change user data')
		#Self.actChangeOther.__roles__ = ["mysqlRole"]
		
		fname = '%s/a_ChangePassword' % dtml_dir
		_add_dtml_method (Self, fname, 'actChangePassword', 'Change user pass')
		#Self.actChangePassword.__roles__ = ["mysqlRole"]
			
		fname = '%s/a_CreateUser' % dtml_dir
		_add_dtml_method (Self, fname, 'actCreateUser', 'Create a user')
		#Self.actCreateUser.__roles__ = None
		
	def _addRoles (Self, ZopeParent):
		if ZopeParent:
			roles = ['mysqlRole']
			if Self._v_user_db:
				roles = roles + Self._v_user_db.list_rolenames ()
			
			util.log_debug ("connect (): Adding roles: " + `roles`)
			
			for role in roles:
				_add_role (ZopeParent, role)

	def _setupDisabledPorts (Self, ports):
		if type (ports) != type (''):
			ports = "%s" % (ports,)

		Self.disable_ports = string.split (string.strip (ports))
		
	# ------------------------
	# connect / disconnect
		
	def __connect (Self):
		util.log_debug ("connect () called.")
		
		Self._v_db = db.dbConnection (Self.mysql_server, 
						Self.mysql_port,
						Self.mysql_socket,
						Self.mysql_db, Self.mysql_user, Self.mysql_password)
						
		Self._v_db.connect ()
		
		ci = vdb.HTTPCookieInfo ()
		ci.path = Self.cookie_path
		ci.domain = Self.cookie_domain
		
		Self._v_token_db = vdb.TokenDb (Self._v_db, Self.realm, ci)
		
		Self._v_user_db = vdb.UserDb (Self._v_db, Self.realm)
		Self._v_session_db = vdb.SessionDb (Self._v_db)
		Self._v_miscdata_db = vdb.MiscDataDB (Self._v_db)
		
	def __disconnect (Self):
		if Self._v_db: Self._v_db.disconnect ()
		
		Self._v_db = None
		
		Self._v_token_db = None
		Self._v_user_db = None
		Self._v_session_db = None
		Self._v_miscdata_db = None
		
	# ------------------------
	# Session/user utility functions
	
	def __terminate_session (Self, REQUEST, RESPONSE, session):
		
		if cfg.SESSION_LOG: 
			log.session_log_end (Self._v_db, Self.realm, session)
			
		Self._v_token_db.request_delete_token (RESPONSE, 
											'session', session.getId ())
	
		REQUEST ['Session'] = None
	
	def __create_session (Self, REQUEST, RESPONSE, user):
		if user: user_id = user.getDBId ()
		else: user_id = cfg.VDB_INVALID_ID

		session = Self._v_session_db.create_session (user_id)
		Self._v_token_db.request_assign_new_token (RESPONSE, 'session', 
			session.getId (), cfg.SESSION_TOKEN_TIMEOUT, cfg.SESSION_TOKEN_LIFE)
		
		session._set_user_folder (Self)
		
		if cfg.SESSION_LOG: 
			log.session_log_begin (Self._v_db, Self.realm, session)
		
		return session

	def __get_session_user (Self, session):
		""" This function returns user object that corresponds to
			a session.
			
			There are three possibilites: Valid user is assigned to a session
			and it belongs to user folder's realm, it does not belong to
			user folder's realm, and there is no valid users assigned to 
			a session.
		"""
	
		if session is None: return None
		
		user_id = session.getUserId ()
	
		if user_id != cfg.VDB_INVALID_ID:
			user = Self._v_user_db.get_user_by_id (user_id)
			
			if user: assert (user.getRealm () == Self.realm)
		else:
			user = None
			
		return user
			
	def __get_session_from_request (Self, REQUEST):
		sess_id = Self._v_token_db.request_check_token (REQUEST, 'session')
		
		session = Self._v_session_db.get_session_by_id (sess_id)
		if session: session._set_user_folder (Self)
		
		return session
		
	def __setup_session (Self, REQUEST, user, session, amethod, target_path):
		""" This method sets up user session.
		
			It assumes user is athenticated/authorized. If user is None,
			then it assumes Anonymous is authorized. Parameter amethod
			represents the way user is authenticated.
		"""
		
		RESPONSE = REQUEST ['RESPONSE']
		
		if session:
			anon_session = session.isAnonymousUser ()
			
			# Is session userid in the current realm ?
			if not anon_session:
				wrong_session_user = 0
				
				if not anon_session:
					suser_id = session.getUserId ()
					
					if user is None: 
						wrong_session_user = 1
					else:
						if user.getDBId () != suser_id: 
							wrong_session_user = 1
			
				if wrong_session_user:
					util.log_debug ("__setup_session (): Wrong session user.")
					
					Self.__terminate_session (REQUEST, RESPONSE, session)
					
					anon_session = 0
					session = None
					
				assert (amethod != 'Cookie')
					# Since there is session user 
					# use_sessions is true, since this function is called
				
		if not session: 
			session = Self.__create_session (REQUEST, RESPONSE, user)
			anon_session = 0
			
		if user and anon_session:
			Self._v_session_db.assign_user_id (session, user.getDBId ())
		
			util.log_debug ("__setup_session (): Assigning " +
						"user '%s' to the session." % user.getUserName ())
			
		REQUEST ['Session'] = session
		if cfg.SESSION_LOG: 
			obj = util.get_obj_from_request (REQUEST)
			addr = REQUEST.get (cfg.REQUEST_RADDR_FIELD, '')
			
			if REQUEST.has_key ('QUERY_STRING'):
				query_string = REQUEST ['QUERY_STRING']
			else:
				query_string = ""
				
			if type (query_string) != type (""):
				query_string = ""
			
			log.session_log_access (Self._v_db, Self.realm, session, 
										target_path, query_string, obj, addr)
		
	# ------------------------
	# User folder's standard interface functions
	# They are all protected by 'Manage Users' permission
	
	def getUserCount (Self):
		""" Returns number of users. """
	
		if Self._v_user_db: return Self._v_user_db.get_user_count ()
		else: return 0
			
	def getRoleCount (Self):
		""" Returns number of roles. """
	
		if Self._v_user_db: return Self._v_user_db.get_role_count ()
		else: return 0
			
	def getUserNames (Self):
		"""Returns a list of user names or [] if no users exist. """
													
		if Self._v_user_db: return Self._v_user_db.list_usernames ()
		else: return []
			
	def getRoleNames (Self):
		"""Returns a list of role names or [] if no roles exist"""
		
		if Self._v_user_db: return Self._v_user_db.list_rolenames ()
		else: return []
		
	def getUsers (Self):
		"""Returns a list of user objects or [] if no users exist"""
		
		if not Self._v_user_db: return []
		
		usernames = Self._v_user_db.list_usernames ()
		users = []
		
		for uname in usernames:
			user = Self._v_user_db.get_user (uname)
			if user: 
				user._set_user_folder (Self)
				users.append (user)
			
		return users
		
	def getUser (Self, username):
		"""Returns the named user object or None if no such user exists"""
		
		if not Self._v_user_db: return None
		user = Self._v_user_db.get_user (username)
		
		if user:
			user._set_user_folder (Self)
			
			# Don't know why, but Zope wants it without acquisition
			#return user.__of__ (Self)
			return user
			
		return None
		
	# ------------------------
	# Authentication / autorization functions
		
	authenticate_user_pass__roles__ = ()
	def authenticate_user_pass (Self, username, password, REQUEST):
		
		user = Self._v_user_db.get_user (username)
		if not user: return None
		
		if user.authenticate_password (password, REQUEST):
			return user
			
		# Since password is wrong, user is refreshed.
		Self._v_user_db.refresh_user (user)
		
		if user.authenticate_password (password, REQUEST):
			return user
			
		return None
		
	authenticate_session__roles__ = ()	
	def authenticate_session (Self, REQUEST):
		session = Self.__get_session_from_request (REQUEST)
		
		user = Self.__get_session_user (session)
		
		if session: 
			util.log_debug ("authenticate_session (): " 
								"Valid session found: %i" % session.getId ())
		else:
			util.log_debug ("authenticate_session (): No valid session.")
			
		if user:
			util.log_debug ("authenticate_session (): Valid user found.")

		return (user, session)

	authenticate_cookie__roles__ = ()
	def authenticate_cookie (Self, REQUEST):
		user_id = Self._v_token_db.request_check_token (REQUEST, 'user')
		if (user_id != cfg.VDB_INVALID_ID):
			user = Self._v_user_db.get_user_by_id (user_id)
		else:
			user = None
			
		return user

	authenticate_http__roles__ = ()		
	def authenticate_http (Self, REQUEST, auth):
		if auth is None: return None
		if auth == '': return None
	
		auth_type = string.lower (string.split (auth) [0])
		if auth_type != 'basic':
			util.log_error ("Unknown http auth type.")
			return None
		
		username, password = tuple (string.split 
							(base64.decodestring (
											string.split (auth) [-1]), ':'))
									
		util.log_debug ("authenticate_http () called: " 
											+ username + " " + password)
											
		return Self.authenticate_user_pass (username, password, REQUEST)

	authenticate__roles__ = ()		
	def authenticate (Self, REQUEST, auth, use_cookies, use_sessions):
		user = None
		session = None
		amethod = None
	
		if use_sessions:
			(user, session) = Self.authenticate_session (REQUEST)
			
		if not use_cookies: user = None
			# We are not using session auth if we are not using cookies.
		
		if user is None:
			# Try standard method
		
			if use_cookies:
				user = Self.authenticate_cookie (REQUEST)
				if user: amethod = 'Cookie'
			else:
				user = Self.authenticate_http (REQUEST, auth)
				if user: amethod = 'HTTP'
		else:
			amethod = 'Session'
			
		if user:
			util.log_debug ("authenticate (): "
									"User authenticated using %s." % amethod)
		else:
			util.log_debug ("authenticate (): User not authenticated.")
			
		return (user, session, amethod)
		
	authorize__roles__ = ()
	def authorize (Self, user, roles, REQUEST):
		util.log_debug ("authorize (): Username: " + 
						user.getUserName () + " Roles: " + `roles` + ".")
	
		v = REQUEST ['PUBLISHED'] 					# the published object
		a, c, n, v = Self._getobcontext(v, REQUEST)
	
		allowed = AccessControl.User.BasicUserFolder.authorize \
										(Self, user, a, c, n, v, roles)
		
		if not allowed:
			util.log_debug ('authorize (): Not authorized.')
			return None
		else:
			util.log_debug ('authorize (): Authorized.')
			return user
		
	def validate (Self, REQUEST, auth = None, roles = []):
		util.log_debug ("validate () called: [%s], auth = '%s'" % (roles,auth))
		# util.log_debug ("%s" % (str (REQUEST), ))

		use_cookies = Self.use_cookies
		use_sessions = Self.use_sessions

		port = util.get_port_from_request (REQUEST)
		
		if (port in Self.disable_ports):
			util.log_debug ("validate (): disabling cookie auth.")
			use_cookies = 0
			use_sessions = 0
			
		if roles is None: roles = [cfg.ZOPE_ANONYMOUS_ROLE, ]
		if auth is None: auth = ''
		
		if cfg.VALIDATE_ALWAYS_SUPER:
			util.log_debug ("Always validating super.")
			return AccessControl.SpecialUsers.super.__of__ (Self)
			
		RESPONSE = REQUEST ['RESPONSE']
		target_path = util.get_path_from_request (REQUEST)
		
		(user, session, method) = \
			Self.authenticate (REQUEST, auth, use_cookies, use_sessions)
		
		if not user:
			# Try anonymous
			if (use_cookies) and (cfg.ZOPE_ANONYMOUS_ROLE in roles):
				util.log_debug ("Validating nobody.")
				
				if use_sessions:
					Self.__setup_session (REQUEST, 
									None, session, method, target_path) 
				
				return AccessControl.SpecialUsers.nobody.__of__ (Self)
		
			# We need to ask for a login
			if use_cookies:
				REQUEST ['login_goto'] = target_path
				raise 'LoginRequired',  Self.docLogin (Self, REQUEST)
			else:
				return None
			
		# We have a valid authenticated user
		
		user._set_user_folder (Self)
		user = Self.authorize (user, roles, REQUEST)
		
		if not user:
			raise 'Forbidden', \
							"You are not authorized to access this document."
							
		if use_sessions: 
			Self.__setup_session (REQUEST, user, session, method, target_path)
		
		if cfg.VALIDATE_ONLY_ANONYMOUS:
			if cfg.ZOPE_ANONYMOUS_ROLE in roles:
				AccessControl.SpecialUsers.nobody.__of__ (Self)
			else:
				raise 'Forbidden', \
					"Only anonymous access is allowed."
		
		return user.__of__ (Self)
		
	# ------------------------
	# Other interface functions.
	#
	# miscdata_ functions () are called from user and session objects.
	# refresh_user () is called from user objects.
		
	create_user__roles__ = ()
	def create_user (Self, username, password, roles, realname, email):
	
		user = Self._v_user_db.create_user (username, roles)
		if not user:
			return 0
					
		user._setAuthData (roles, [])
					
		Self._v_user_db.change_user_password (user, password)
		Self._v_user_db.change_user_other (user, realname, email)
			
		return 1
		
	check_valid_mysql_user__roles__ = None
	def check_valid_mysql_user (Self, REQUEST):
		if not REQUEST.has_key ('AUTHENTICATED_USER'):
			raise 'Unauthorized'
			
		user = REQUEST ['AUTHENTICATED_USER']
		try:
			user.isAmysqlUserObject
		except AttributeError:
			raise 'NotAmysqlUser'
			
		if user.getRealm () != Self.realm:
			raise 'InvalidUser'
			
		return user
		
	miscdata_set__roles__ = ()	
	def miscdata_set (Self, obj_class, obj_id, misc_id, misc_tuple):
		Self._v_miscdata_db.save_data (obj_class, obj_id, misc_id, misc_tuple)
	
	miscdata_del__roles__ = ()	
	def miscdata_del (Self, obj_class, obj_id, misc_id):
		Self._v_miscdata_db.del_data (obj_class, obj_id, misc_id)
		
	miscdata_get__roles__ = ()
	def miscdata_get (Self, obj_class, obj_id, misc_id):
		return Self._v_miscdata_db.get_data (obj_class, obj_id, misc_id)
		
	refresh_user__roles__ = ()
	def refresh_user (Self, user):
		return Self._v_user_db.refresh_user (user)
		
	getDBFieldSize__roles__ = None
	def getDBFieldSize (Self, fname):
		n = "FSIZE_" + fname
		val = cfg.__dict__ [n]

		return val

	getSpecialRoles__roles__ = None
	def getSpecialRoles (Self):
		return cfg.ZOPE_SYSTEM_ROLES

	UserFolderUsesCookies__roles__ = None
	def UserFolderUsesCookies (Self):
		return Self.use_cookies

	# ------------------------
	# Manage functions

	def manage_addMySQLRoles (Self):
		""" Add mysql roles to the parent folder. """
	
		parent = Self.sq_parent
	
		if parent:
			roles = Self._v_user_db.list_rolenames ()
			util.log_debug ("connect (): Adding roles: " + `roles`)
			
			for role in roles:
				parent._addRole (role)
		
	def manage_edit (Self, realm, use_cookies, server, port, socket, database, 
						username, password, anon_create_ok, 
						anon_create_role, dtml_user_dir, use_sessions,
						cookie_path, cookie_domain, 
						disable_cookies_for_ports,
						REQUEST):
		""" Change properties. """
		
		if not cookie_path: cookie_path = '/'
		
		Self.__disconnect ()
			
		Self.mysql_server = server
		Self.mysql_port = port
		Self.mysql_socket = string.strip (socket)
		Self.mysql_db = database
		
		Self.mysql_user = username
		Self.mysql_password = password
		
		Self.anon_create_ok = anon_create_ok
		Self.anon_create_role = anon_create_role
		
		Self.realm = realm
		Self.use_cookies = use_cookies
		Self.use_sessions = use_sessions
		
		Self.cookie_path = cookie_path
		Self.cookie_domain = cookie_domain
		
		Self.dtml_user_dir = dtml_user_dir
		
		Self.__connect ()

		Self._setupDisabledPorts (disable_cookies_for_ports)
		
		if REQUEST:
			return Globals.MessageDialog(
					title  ='mysqlUserFolder Changed',
					message='mysqlUserFolder properties have been updated',
					action =REQUEST['URL1']+'/manage_properties',
					target ='manage_main')
					
	def manage_connect (Self, REQUEST):
		""" Reconnects to server """
		
		Self.__disconnect ()
		Self.__connect ()
		
		if REQUEST:
			return Globals.MessageDialog(
					title  ='mysqlUserFolder Reconnected',
					message='mysqlUserFolder is reconnected to server',
					action =REQUEST['URL1']+'/manage_properties',
					target ='manage_main')
		
	def manage_users_create (Self, REQUEST, username, password, role,
														realname, email):
		""" Creates new user (managment).

			Parameter role can be typle, list or single role.
			This parameter is not called roles because of backward
			compatibility.		
		"""

		if type(role) == type((None,)):
			roles = list(role)
		elif type (role) == type ([]):
			roles = role
		else:
			roles = [role]
		
		ok = Self.create_user (username, password, roles, realname, email)
												
		if not ok:
			if REQUEST:
				return Globals.MessageDialog(
					title  ='User exists',
					message='Error: Specified user already exists !',
					action =REQUEST['URL1']+'/manage_users',
					target ='manage_main')
			else:
				return 0
		
		if REQUEST:			
			return Globals.MessageDialog(
				title  ='User created',
				message='Specified user is created.',
				action =REQUEST['URL1']+'/manage_users',
				target ='manage_main')
		else:
			return 1
				
	def manage_users_change_roles (Self, REQUEST, username, roles):
		""" Changes user roles (managment) """

		if type (roles) != type ([]): roles = [roles]
	
		user = Self.getUser (username)
		if not user:
			if REQUEST:
				return Globals.MessageDialog(
					title  ='User does not exist',
					message='Error: Specified user does not exist.',
					action =REQUEST['URL1']+'/manage_users',
					target ='manage_main')
			else:
				return 0
				
		ret = Self._v_user_db.change_user_roles (user, roles)

		if not ret:
			if REQUEST:
				return Globals.MessageDialog(
					title  ='DB error',
					message='Error: DB error.',
					action =REQUEST['URL1']+'/manage_users',
					target ='manage_main')
			else:
				return 0
			
		
		if REQUEST:
			return Globals.MessageDialog(
				title  ='User roles changed',
				message='User roles are changed.',
				action =REQUEST['URL1']+'/manage_users',
				target ='manage_main')
		else:
			return 1

	def manage_users_change_other (Self, REQUEST, username, realname, email):
		""" Changes user data (managment) """
	
		user = Self.getUser (username)
		if not user:
			if REQUEST:
				return Globals.MessageDialog(
					title  ='User does not exist',
					message='Error: Specified user does not exist.',
					action =REQUEST['URL1']+'/manage_users',
					target ='manage_main')
			else:
				return 0
				
		if realname == '': realname = user.getRealName ()
		if email == '': email = user.getEMail ()
				
		Self._v_user_db.change_user_other (user, realname, email)
		
		if REQUEST:
			return Globals.MessageDialog(
				title  ='User changed',
				message='User data are changed.',
				action =REQUEST['URL1']+'/manage_users',
				target ='manage_main')
		else:
			return 1
			
	def manage_users_change_password (Self, REQUEST, username, password):
		""" Changes user data (managment) """
	
		user = Self.getUser (username)
		if not user:
			if REQUEST:
				return Globals.MessageDialog(
					title  ='User does not exist',
					message='Error: Specified user does not exist.',
					action =REQUEST['URL1']+'/manage_users',
					target ='manage_main')
			else:
				return 0
				
		Self._v_user_db.change_user_password (user, password)
		
		if REQUEST:
			return Globals.MessageDialog(
				title  ='User changed',
				message='User password is changed.',
				action =REQUEST['URL1']+'/manage_users',
				target ='manage_main')
		else:
			return 1
			
	def manage_users_delete (Self, REQUEST, username):
		""" Deletes user (managment) """
	
		ok = Self._v_user_db.del_user (username)
		
		if not ok:
		
			if REQUEST:
				return Globals.MessageDialog(
					title  ='Error deleting user',
					message='Specified user is not deleted.',
					action =REQUEST['URL1']+'/manage_users',
					target ='manage_main')
			else:
				return 0

		if REQUEST:		
			return Globals.MessageDialog(
				title  ='User deleted',
				message='Specified user is deleted.',
				action =REQUEST['URL1']+'/manage_users',
				target ='manage_main')
		else:
			return 1

	# ------------------------
	# "user" methods
		
	user_login__roles__ = None
	def user_login (Self, REQUEST, RESPONSE, username, password, 
						create_global = 0, login_goto_path = "docUserPage"):
		""" Authenticates user """
		
		if not Self.use_cookies:
			raise 'Forbidden', \
						"You are not authorized to access this document."
						
		user = Self.authenticate_user_pass (username, password, REQUEST)
		if not user:
			if Self.use_sessions:
				if cfg.SESSION_LOG:
					session = Self.__get_session_from_request (REQUEST)
					addr = REQUEST.get (cfg.REQUEST_RADDR_FIELD, '')
					
					# session can be None
					log.session_log_login_failed (Self._v_db, 
							Self.realm, session, username, password, addr)
			
			REQUEST ['login_goto'] = login_goto_path
			return Self.docLogin (Self, REQUEST, login_failed = 1)
		
		session = None
		if Self.use_sessions:
			session = Self.__get_session_from_request (REQUEST)
			
			if session:
				if not session.isAnonymousUser ():
					Self.__terminate_session (REQUEST, RESPONSE, session)
					session = None
				
		util.log_debug ("user_login (): user authentificated.")
		
		Self._v_token_db.request_assign_new_token (RESPONSE, 
								'user', user.getDBId (),
								cfg.USER_TOKEN_TIMEOUT, cfg.USER_TOKEN_LIFE)
								
		if session:
			assert (session.getUserId () == cfg.VDB_INVALID_ID)
			
			if cfg.SESSION_LOG: 
				addr = REQUEST.get (cfg.REQUEST_RADDR_FIELD, '')
				log.session_log_login (Self._v_db, Self.realm, 
													session, username, addr)
		
		RESPONSE.redirect (login_goto_path)
		
	user_logout__roles__ = ['mysqlRole']
	def user_logout (Self, REQUEST, RESPONSE, logout_goto_path = "/"):
		""" Terminates user session """
		
		if not Self.use_cookies:
			raise 'Forbidden', \
						"You are not authorized to access this document."
						
		user = Self.check_valid_mysql_user (REQUEST)
						
		if Self.use_sessions:
			session = Self.__get_session_from_request (REQUEST)
			if session:
				Self.__terminate_session (REQUEST, RESPONSE, session)
			
		Self._v_token_db.request_delete_token (RESPONSE, 
													'user', user.getDBId ())
		
		return RESPONSE.redirect (logout_goto_path)
	
	#user_change_other_data__roles__ = ['mysqlRole']
	user_change_other_data__roles__ = None
	def user_change_other_data (Self, REQUEST, realname, email):
		""" Change user's other data """
		
		user = Self.check_valid_mysql_user (REQUEST)
		
		if (user.getRealName () == realname) and (user.getEMail () == email):
			return 1
		
		ret = Self._v_user_db.change_user_other (user, realname, email)
		return ret
	
	#user_change_password__roles__ = ['mysqlRole']
	user_change_password__roles__ = None
	def user_change_password (Self, REQUEST, new_pass):
		""" Changes user's password """
		
		user = Self.check_valid_mysql_user (REQUEST)
		ret = Self._v_user_db.change_user_password (user, new_pass)
		return ret

	user_create__roles__ = None
	def user_create (Self, REQUEST, username, realname, email, password):
		""" Anonymous creation """
		
		if not Self.anon_create_ok:
			raise 'Forbidden', \
						"You are not authorized to access this document."
		
		roles = [Self.anon_create_role]
				
		ok = Self.create_user (username, password, roles, realname, email)
		return ok
	
Globals.default__class_init__ (mysqlUserFolder)

